import unittest

from src.simple_nlp_library.preprocessing import single_spaces, lower_letters, non_stopword_tokens, \
    semantic_tokens


class TestPreprocessing(unittest.TestCase):
    def test_single_spaces(self):
        self.assertEqual(
            single_spaces('The  quick \t brown \n fox jumps'),
            'The quick brown fox jumps'
        )

    def test_lower_letters(self):
        self.assertEqual(
            lower_letters('The 2quick brown fox jumps!'),
            'the quick brown fox jumps'
        )

    def test_non_stopword_tokens(self):
        self.assertEqual(
            non_stopword_tokens(['the', 'quick', 'brown', 'fox', 'jumps']),
            ['quick', 'brown', 'fox', 'jumps']
        )

    def test_semantic_tokens(self):
        self.assertEqual(
            semantic_tokens('The 2 quick \t brown fox jumps, over the lazy dog!'),
            ['quick', 'brown', 'fox', 'jumps', 'lazy', 'dog']
        )



from typing import List


def lower_letters(text: str) -> str:
    return ''.join([c.lower() for c in text if c.isalpha() or c.isspace()])


def single_spaces(text: str) -> str:
    return ' '.join(text.split())


def non_stopword_tokens(tokens: List[str]) -> List[str]:
    stop_words = ['i', 'me', 'my', 'myself', 'we', 'our', 'ours', 'ourselves', 'you', 'youre', 'youve', 'youll', 'youd', 'your', 'yours', 'yourself', 'yourselves', 'he', 'him', 'his', 'himself', 'she', 'shes', 'her', 'hers', 'herself', 'it', 'its', 'its', 'itself', 'they', 'them', 'their', 'theirs', 'themselves', 'what', 'which', 'who', 'whom', 'this', 'that', 'thatll', 'these', 'those', 'am', 'is', 'are', 'was', 'were', 'be', 'been', 'being', 'have', 'has', 'had', 'having', 'do', 'does', 'did', 'doing', 'a', 'an', 'the', 'and', 'but', 'if', 'or', 'because', 'as', 'until', 'while', 'of', 'at', 'by', 'for', 'with', 'about', 'against', 'between', 'into', 'through', 'during', 'before', 'after', 'above', 'below', 'to', 'from', 'up', 'down', 'in', 'out', 'on', 'off', 'over', 'under', 'again', 'further', 'then', 'once', 'here', 'there', 'when', 'where', 'why', 'how', 'all', 'any', 'both', 'each', 'few', 'more', 'most', 'other', 'some', 'such', 'no', 'nor', 'not', 'only', 'own', 'same', 'so', 'than', 'too', 'very', 's', 't', 'can', 'will', 'just', 'don', 'dont', 'should', 'shouldve', 'now', 'd', 'll', 'm', 'o', 're', 've', 'y', 'ain', 'aren', 'arent', 'couldn', 'couldnt', 'didn', 'didnt', 'doesn', 'doesnt', 'hadn', 'hadnt', 'hasn', 'hasnt', 'haven', 'havent', 'isn', 'isnt', 'ma', 'mightn', 'mightnt', 'mustn', 'mustnt', 'needn', 'neednt', 'shan', 'shant', 'shouldn', 'shouldnt', 'wasn', 'wasnt', 'weren', 'werent', 'won', 'wont', 'wouldn', 'wouldnt']
    return [token for token in tokens if token not in dict.fromkeys(stop_words)]


def semantic_tokens(text: str) -> List[str]:
    text = lower_letters(text)
    text = single_spaces(text)
    tokens = non_stopword_tokens(text.split())
    return tokens



import unittest

from src.simple_nlp_library.metrics import dot_product, frobenius_norm, cosine_similarity, inserting_distance, \
    inserting_similarity


class TestMetrics(unittest.TestCase):
    def test_dot_product(self):
        self.assertEqual(dot_product([1.0, 2.0, 3.0], [4.0, 5.0, 6.0]), 32)

    def test_frobenius_norm(self):
        self.assertEqual(frobenius_norm([1.0, 1.0, 1.0, 2.0, 3.0]), 4.0)

    def test_cosine_similarity(self):
        self.assertAlmostEqual(cosine_similarity([1.0, 1.0, 1.0], [4.0, 4.0, 4.0]), 1.0, 1)

    def test_inserting_distance(self):
        self.assertEqual(inserting_distance('simple', 'simple'), 0)
        self.assertEqual(inserting_distance('simple', 'simpler'), 1)
        self.assertEqual(inserting_distance('easy', 'hard'), 4)

    def test_inserting_similarity(self):
        self.assertEqual(inserting_similarity('simple', 'simple'), 1.0)
        self.assertEqual(inserting_similarity('simple', 'simpler'), 1.0 - 1 / 7)
        self.assertEqual(inserting_similarity('easy', 'hard'), 0.0)
